@@ -9,7 +9,7 @@
 */
 
 /* ====================================================================
- * Copyright (c) 1998-2001 Ralf S. Engelschall. All rights reserved.
+ * Copyright (c) 1998-2006 Ralf S. Engelschall. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -450,7 +450,16 @@ const char *ssl_cmd_SSLCryptoDevice(
     SSLModConfigRec *mc = myModConfig();
     const char *err;
     ENGINE *e;
+#if SSL_LIBRARY_VERSION >= 0x00907000
+    static int loaded_engines = FALSE;
 
+    /* early loading to make sure the engines are already 
+       available for ENGINE_by_id() above... */
+    if (!loaded_engines) {
+        ENGINE_load_builtin_engines();
+        loaded_engines = TRUE;
+    }
+#endif
     if ((err = ap_check_cmd_context(cmd, GLOBAL_ONLY)) != NULL)
         return err;
     if (strcEQ(arg, "builtin")) {
@@ -747,7 +756,7 @@ const char *ssl_cmd_SSLSessionCache(
                 return "SSLSessionCache: Invalid argument: no closing parenthesis";
             *cp2 = NUL;
             mc->nSessionCacheDataSize = atoi(cp);
-            if (mc->nSessionCacheDataSize <= 8192)
+            if (mc->nSessionCacheDataSize < 8192)
                 return "SSLSessionCache: Invalid argument: size has to be >= 8192 bytes";
             maxsize = ap_mm_core_maxsegsize();
             if (mc->nSessionCacheDataSize >= maxsize)
@@ -769,7 +778,7 @@ const char *ssl_cmd_SSLSessionCache(
                 return "SSLSessionCache: Invalid argument: no closing parenthesis";
             *cp2 = NUL;
             mc->nSessionCacheDataSize = atoi(cp);
-            if (mc->nSessionCacheDataSize <= 8192)
+            if (mc->nSessionCacheDataSize < 8192)
                 return "SSLSessionCache: Invalid argument: size has to be >= 8192 bytes";
             maxsize = ap_mm_core_maxsegsize();
             if (mc->nSessionCacheDataSize >= maxsize)
